#include <cmath>
#include <fstream>
#include <iostream>
#include <iomanip>
#include <string>
#include <vector>
using namespace std;

#include "Timer.h"
#include "FastMath.h"

void usage(void)
{
  cout << "Usage : test_fasmath [fichier-de-sortie]" << endl;
}

int main(int argc, char* argv[])
{
  string file("fastmath.out");
  switch(argc)
  {
    case 1:break;
    case 2:file = string(argv[1]);break;
    default:usage();break;
  }

  ofstream fResults(file.c_str());
  fResults << "\tnormal(float)\tnormal(double)\tfast(float)\tfast(double)\t(float)fast(double)\n";

  Timer timer;
  cout << "remplissage des tableaux..." << flush;
  const size_t taille = 20000000;
  vector<float>  vf;
  vector<double> vd;
  vf.reserve(taille);
  vd.reserve(taille);
  srand(0);
  for(size_t i=0 ; i<taille ; ++i)
    vf.push_back(100*static_cast<float>(rand())/RAND_MAX);
  srand(0);
  for(size_t i=0 ; i<taille ; ++i)
    vd.push_back(100*static_cast<double>(rand())/RAND_MAX);
  cout << "...ok !" << endl << endl;

  float  yf = 0;
  double yd = 0;
  
  double values[22] = {0,
                       0.01, 0.02, 0.03, 0.04, 0.05,
                       0.1, 0.2, 0.3, 0.4, 0.5,
                       1.0, 2.0, 3.0, 4.0, 5.0,
                       10.0, 20.0, 30.0, 40.0, 50.0,
                       100.0};

  double valuesTrigo[21] = {-1, -0.9, -0.8, -0.7, -0.6, -0.5, -0.4, -0.3, -0.2, -0.1,
                            0,
                            0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0};

  //Sqrt
  cout << setw(10) << "x" << '\t'
       << setw(10) << "sqrt(double x)" << '\t'
       << setw(10) << "fastSqrt(float x)" << '\t'
       << setw(10) << "fastSqrt(double x)" << endl;
  for(unsigned int i = 0 ; i<sizeof(values)/sizeof(double) ; ++i)
    cout << setw(10) << values[i] << '\t'
         << setw(10) << sqrt(values[i]) << '\t'
         << setw(10) << FastMath::fastSqrt(static_cast<float>(values[i])) << '\t'
         << setw(10) << FastMath::fastSqrt(values[i]) << endl;

  fResults << "\nsqrt";
  cout << "Benchmark sqrtf(float) (" << taille << " elements)" << endl;
  timer.start();  
  yf = 0;
  for(vector<float>::const_iterator it = vf.begin() ; it != vf.end() ; ++it)
    yf += sqrtf(*it);
  timer.stop();
  cout << timer << "\t(computed result = " << yf << ")" << endl;
  fResults << '\t' << timer.value();
  
  cout << "Benchmark sqrt(double) (" << taille << " elements)" << endl;
  timer.start();  
  yd = 0;
  for(vector<double>::const_iterator it = vd.begin() ; it != vd.end() ; ++it)
    yd += sqrt(*it);
  timer.stop();
  cout << timer << "\t(computed result = " << yd << ")" << endl;
  fResults << '\t' << timer.value();
  
  cout << "Benchmark fastSqrt(float) (" << taille << " elements)" << endl;
  timer.start();  
  yf = 0;
  for(vector<float>::const_iterator it = vf.begin() ; it != vf.end() ; ++it)
    yf += FastMath::fastSqrt(*it);
  timer.stop();
  cout << timer << "\t(computed result = " << yf << ")" << endl;
  fResults << '\t' << timer.value();
  
  cout << "Benchmark fastSqrt(double) (" << taille << " elements)" << endl;
  timer.start();  
  yd = 0;
  for(vector<double>::const_iterator it = vd.begin() ; it != vd.end() ; ++it)
    yd += FastMath::fastSqrt(*it);
  timer.stop();
  cout << timer << "\t(computed result = " << yd << ")" << endl;
  fResults << '\t' << timer.value();
  
  cout << "Benchmark (float)fastSqrt(double) (" << taille << " elements)" << endl;
  timer.start();  
  yf = 0;
  for(vector<double>::const_iterator it = vd.begin() ; it != vd.end() ; ++it)
    yf += (float) FastMath::fastSqrt(*it);
  timer.stop();
  cout << timer << "\t(computed result = " << yf << ")" << endl;
  fResults << '\t' << timer.value();

  //InvSqrt
  cout << setw(10) << "x" << '\t'
       << setw(10) << "1/sqrt(double x)" << '\t'
       << setw(10) << "fastInvSqrt(float x)" << '\t'
       << setw(10) << "fastinvSqrt(double x)" << endl;
  for(unsigned int i = 0 ; i<sizeof(values)/sizeof(double) ; ++i)
    cout << setw(10) << values[i] << '\t'
         << setw(10) << 1/sqrt(values[i]) << '\t'
         << setw(10) << FastMath::fastInvSqrt(static_cast<float>(values[i])) << '\t'
         << setw(10) << FastMath::fastInvSqrt(values[i]) << endl;

  fResults << "\n1/sqrt";
  cout << "Benchmark 1/sqrtf(float) (" << taille << " elements)" << endl;
  timer.start();  
  yf = 0;
  for(vector<float>::const_iterator it = vf.begin() ; it != vf.end() ; ++it)
    yf += 1.0f/sqrtf(*it);
  timer.stop();
  cout << timer << "\t(computed result = " << yf << ")" << endl;
  fResults << '\t' << timer.value();
  
  cout << "Benchmark 1/sqrt(double) (" << taille << " elements)" << endl;
  timer.start();  
  yd = 0;
  for(vector<double>::const_iterator it = vd.begin() ; it != vd.end() ; ++it)
    yd += 1.0/sqrt(*it);
  timer.stop();
  cout << timer << "\t(computed result = " << yd << ")" << endl;
  fResults << '\t' << timer.value();
  
  cout << "Benchmark fastInvSqrt(float) (" << taille << " elements)" << endl;
  timer.start();  
  yf = 0;
  for(vector<float>::const_iterator it = vf.begin() ; it != vf.end() ; ++it)
    yf += FastMath::fastInvSqrt(*it);
  timer.stop();
  cout << timer << "\t(computed result = " << yf << ")" << endl;
  fResults << '\t' << timer.value();
  
  cout << "Benchmark fastInvSqrt(double) (" << taille << " elements)" << endl;
  timer.start();  
  yd = 0;
  for(vector<double>::const_iterator it = vd.begin() ; it != vd.end() ; ++it)
    yd += FastMath::fastInvSqrt(*it);
  timer.stop();
  cout << timer << "\t(computed result = " << yd << ")" << endl;
  fResults << '\t' << timer.value();
  
  cout << "Benchmark (float)fastInvSqrt(double) (" << taille << " elements)" << endl;
  timer.start();  
  yf = 0;
  for(vector<double>::const_iterator it = vd.begin() ; it != vd.end() ; ++it)
    yf += (float) FastMath::fastInvSqrt(*it);
  timer.stop();
  cout << timer << "\t(computed result = " << yf << ")" << endl;
  fResults << '\t' << timer.value();

  //Sin
  cout << setw(10) << "x" << '\t'
       << setw(10) << "sin(double x)" << '\t'
       << setw(10) << "fastSin(float x)" << '\t'
       << setw(10) << "fastSin(double x)" << endl;
  for(unsigned int i = 0 ; i<sizeof(values)/sizeof(double) ; ++i)
    cout << setw(10) << values[i] << '\t'
         << setw(10) << sin(values[i]) << '\t'
         << setw(10) << FastMath::fastSin(static_cast<float>(values[i])) << '\t'
         << setw(10) << FastMath::fastSin(values[i]) << endl;

  fResults << "\nsin";
  cout << "Benchmark sinf(float) (" << taille << " elements)" << endl;
  timer.start();  
  yf = 0;
  for(vector<float>::const_iterator it = vf.begin() ; it != vf.end() ; ++it)
    yf += sinf(*it);
  timer.stop();
  cout << timer << "\t(computed result = " << yf << ")" << endl;
  fResults << '\t' << timer.value();
  
  cout << "Benchmark sin(double) (" << taille << " elements)" << endl;
  timer.start();  
  yd = 0;
  for(vector<double>::const_iterator it = vd.begin() ; it != vd.end() ; ++it)
    yd += sin(*it);
  timer.stop();
  cout << timer << "\t(computed result = " << yd << ")" << endl;
  fResults << '\t' << timer.value();
    
  cout << "Benchmark fastSin(float) (" << taille << " elements)" << endl;
  timer.start();  
  yf = 0;
  for(vector<float>::const_iterator it = vf.begin() ; it != vf.end() ; ++it)
    yf += FastMath::fastSin(*it);
  timer.stop();
  cout << timer << "\t(computed result = " << yf << ")" << endl;
  fResults << '\t' << timer.value();

  cout << "Benchmark fastSin(double) (" << taille << " elements)" << endl;
  timer.start();  
  yd = 0;
  for(vector<double>::const_iterator it = vd.begin() ; it != vd.end() ; ++it)
    yd += FastMath::fastSin(*it);
  timer.stop();
  cout << timer << "\t(computed result = " << yd << ")" << endl;
  fResults << '\t' << timer.value();

  cout << "Benchmark (float)fastSin(double) (" << taille << " elements)" << endl;
  timer.start();  
  yf = 0;
  for(vector<double>::const_iterator it = vd.begin() ; it != vd.end() ; ++it)
    yf += (float) FastMath::fastSin(*it);
  timer.stop();
  cout << timer << "\t(computed result = " << yf << ")" << endl;
  fResults << '\t' << timer.value();
  
  //Cos
  cout << setw(10) << "x" << '\t'
       << setw(10) << "cos(double x)" << '\t'
       << setw(10) << "fastCos(float x)" << '\t'
       << setw(10) << "fastCos(double x)" << endl;
  for(unsigned int i = 0 ; i<sizeof(values)/sizeof(double) ; ++i)
    cout << setw(10) << values[i] << '\t'
         << setw(10) << cos(values[i]) << '\t'
         << setw(10) << FastMath::fastCos(static_cast<float>(values[i])) << '\t'
         << setw(10) << FastMath::fastCos(values[i]) << endl;

  fResults << "\ncos";
  cout << "Benchmark cosf(float) (" << taille << " elements)" << endl;
  timer.start();  
  yf = 0;
  for(vector<float>::const_iterator it = vf.begin() ; it != vf.end() ; ++it)
    yf += cosf(*it);
  timer.stop();
  cout << timer << "\t(computed result = " << yf << ")" << endl;
  fResults << '\t' << timer.value();
  
  cout << "Benchmark cos(double) (" << taille << " elements)" << endl;
  timer.start();  
  yd = 0;
  for(vector<double>::const_iterator it = vd.begin() ; it != vd.end() ; ++it)
    yd += cos(*it);
  timer.stop();
  cout << timer << "\t(computed result = " << yd << ")" << endl;
  fResults << '\t' << timer.value();
  
  cout << "Benchmark fastCos(float) (" << taille << " elements)" << endl;
  timer.start();  
  yf = 0;
  for(vector<float>::const_iterator it = vf.begin() ; it != vf.end() ; ++it)
    yf += FastMath::fastCos(*it);
  timer.stop();
  cout << timer << "\t(computed result = " << yf << ")" << endl;
  fResults << '\t' << timer.value();
  
  cout << "Benchmark fastCos(double) (" << taille << " elements)" << endl;
  timer.start();  
  yd = 0;
  for(vector<double>::const_iterator it = vd.begin() ; it != vd.end() ; ++it)
    yd += FastMath::fastCos(*it);
  timer.stop();
  cout << timer << "\t(computed result = " << yd << ")" << endl;
  fResults << '\t' << timer.value();
  
  cout << "Benchmark (float)fastCos(double) (" << taille << " elements)" << endl;
  timer.start();  
  yf = 0;
  for(vector<double>::const_iterator it = vd.begin() ; it != vd.end() ; ++it)
    yf += (float) FastMath::fastCos(*it);
  timer.stop();
  cout << timer << "\t(computed result = " << yf << ")" << endl;
  fResults << '\t' << timer.value();

  //Tan
  cout << setw(10) << "x" << '\t'
       << setw(10) << "tan(double x)" << '\t'
       << setw(10) << "fastTan(float x)" << '\t'
       << setw(10) << "fastTan(double x)" << endl;
  for(unsigned int i = 0 ; i<sizeof(values)/sizeof(double) ; ++i)
    cout << setw(10) << values[i] << '\t'
         << setw(10) << tan(values[i]) << '\t'
         << setw(10) << FastMath::fastTan(static_cast<float>(values[i])) << '\t'
         << setw(10) << FastMath::fastTan(values[i]) << endl;

  fResults << "\ntan";
  cout << "Benchmark tanf(float) (" << taille << " elements)" << endl;
  timer.start();  
  yf = 0;
  for(vector<float>::const_iterator it = vf.begin() ; it != vf.end() ; ++it)
    yf += tanf(*it);
  timer.stop();
  cout << timer << "\t(computed result = " << yf << ")" << endl;
  fResults << '\t' << timer.value();
  
  cout << "Benchmark tan(double) (" << taille << " elements)" << endl;
  timer.start();  
  yd = 0;
  for(vector<double>::const_iterator it = vd.begin() ; it != vd.end() ; ++it)
    yd += tan(*it);
  timer.stop();
  cout << timer << "\t(computed result = " << yd << ")" << endl;
  fResults << '\t' << timer.value();
  
  cout << "Benchmark fastTan(float) (" << taille << " elements)" << endl;
  timer.start();  
  yf = 0;
  for(vector<float>::const_iterator it = vf.begin() ; it != vf.end() ; ++it)
    yf += FastMath::fastTan(*it);
  timer.stop();
  cout << timer << "\t(computed result = " << yf << ")" << endl;
  fResults << '\t' << timer.value();
  
  cout << "Benchmark fastTan(double) (" << taille << " elements)" << endl;
  timer.start();  
  yd = 0;
  for(vector<double>::const_iterator it = vd.begin() ; it != vd.end() ; ++it)
    yd += FastMath::fastTan(*it);
  timer.stop();
  cout << timer << "\t(computed result = " << yd << ")" << endl;
  fResults << '\t' << timer.value();
  
  cout << "Benchmark (float)fastTan(double) (" << taille << " elements)" << endl;
  timer.start();  
  yf = 0;
  for(vector<double>::const_iterator it = vd.begin() ; it != vd.end() ; ++it)
    yf += (float) FastMath::fastTan(*it);
  timer.stop();
  cout << timer << "\t(computed result = " << yf << ")" << endl;
  fResults << '\t' << timer.value();
  
  //ASin
  cout << setw(10) << "x" << '\t'
       << setw(10) << "asin(double x)" << '\t'
       << setw(10) << "fasASin(float x)" << '\t'
       << setw(10) << "fastASin(double x)" << endl;
  for(unsigned int i = 0 ; i<sizeof(valuesTrigo)/sizeof(double) ; ++i)
    cout << setw(10) << valuesTrigo[i] << '\t'
         << setw(10) << asin(valuesTrigo[i]) << '\t'
         << setw(10) << FastMath::fastASin(static_cast<float>(valuesTrigo[i])) << '\t'
         << setw(10) << FastMath::fastASin(valuesTrigo[i]) << endl;

  fResults << "\nasin";
  cout << "Benchmark asinf(float) (" << taille << " elements)" << endl;
  timer.start();  
  yf = 0;
  for(vector<float>::const_iterator it = vf.begin() ; it != vf.end() ; ++it)
    yf += asinf(*it/50.0f-1.0f);
  timer.stop();
  cout << timer << "\t(computed result = " << yf << ")" << endl;
  fResults << '\t' << timer.value();
  
  cout << "Benchmark asin(double) (" << taille << " elements)" << endl;
  timer.start();  
  yd = 0;
  for(vector<double>::const_iterator it = vd.begin() ; it != vd.end() ; ++it)
    yd += asin(*it/50.0-1.0);
  timer.stop();
  cout << timer << "\t(computed result = " << yd << ")" << endl;
  fResults << '\t' << timer.value();
  
  cout << "Benchmark fastASin(float) (" << taille << " elements)" << endl;
  timer.start();  
  yf = 0;
  for(vector<float>::const_iterator it = vf.begin() ; it != vf.end() ; ++it)
    yf += FastMath::fastASin(*it/50.0f-1.0f);
  timer.stop();
  cout << timer << "\t(computed result = " << yf << ")" << endl;
  fResults << '\t' << timer.value();

  cout << "Benchmark fastASin(double) (" << taille << " elements)" << endl;
  timer.start();  
  yd = 0;
  for(vector<double>::const_iterator it = vd.begin() ; it != vd.end() ; ++it)
    yd += FastMath::fastASin(*it/50.0-1.0);
  timer.stop();
  cout << timer << "\t(computed result = " << yd << ")" << endl;
  fResults << '\t' << timer.value();

  cout << "Benchmark (float)fastASin(double) (" << taille << " elements)" << endl;
  timer.start();  
  yf = 0;
  for(vector<double>::const_iterator it = vd.begin() ; it != vd.end() ; ++it)
    yf += (float) FastMath::fastASin(*it/50.0-1.0);
  timer.stop();
  cout << timer << "\t(computed result = " << yf << ")" << endl;
  fResults << '\t' << timer.value();

  //ACos
  cout << setw(10) << "x" << '\t'
       << setw(10) << "acos(double x)" << '\t'
       << setw(10) << "fastACos(float x)" << '\t'
       << setw(10) << "fastACos(double x)" << endl;
  for(unsigned int i = 0 ; i<sizeof(valuesTrigo)/sizeof(double) ; ++i)
    cout << setw(10) << valuesTrigo[i] << '\t'
         << setw(10) << acos(valuesTrigo[i]) << '\t'
         << setw(10) << FastMath::fastACos(static_cast<float>(valuesTrigo[i])) << '\t'
         << setw(10) << FastMath::fastACos(valuesTrigo[i]) << endl;

  fResults << "\nacos";
  cout << "Benchmark acosf(float) (" << taille << " elements)" << endl;
  timer.start();  
  yf = 0;
  for(vector<float>::const_iterator it = vf.begin() ; it != vf.end() ; ++it)
    yf += acosf(*it/50.0f-1.0f);
  timer.stop();
  cout << timer << "\t(computed result = " << yf << ")" << endl;
  fResults << '\t' << timer.value();
  
  cout << "Benchmark acos(double) (" << taille << " elements)" << endl;
  timer.start();  
  yd = 0;
  for(vector<double>::const_iterator it = vd.begin() ; it != vd.end() ; ++it)
    yd += acos(*it/50.0-1.0);
  timer.stop();
  cout << timer << "\t(computed result = " << yd << ")" << endl;
  fResults << '\t' << timer.value();
  
  cout << "Benchmark fastACos(float) (" << taille << " elements)" << endl;
  timer.start();  
  yf = 0;
  for(vector<float>::const_iterator it = vf.begin() ; it != vf.end() ; ++it)
    yf += FastMath::fastACos(*it/50.0f-1.0f);
  timer.stop();
  cout << timer << "\t(computed result = " << yf << ")" << endl;
  fResults << '\t' << timer.value();
  
  cout << "Benchmark fastACos(double) (" << taille << " elements)" << endl;
  timer.start();  
  yd = 0;
  for(vector<double>::const_iterator it = vd.begin() ; it != vd.end() ; ++it)
    yd += FastMath::fastACos(*it/50.0-1.0);
  timer.stop();
  cout << timer << "\t(computed result = " << yd << ")" << endl;
  fResults << '\t' << timer.value();
  
  cout << "Benchmark (float)fastACos(double) (" << taille << " elements)" << endl;
  timer.start();  
  yf = 0;
  for(vector<double>::const_iterator it = vd.begin() ; it != vd.end() ; ++it)
    yf += (float)FastMath::fastACos(*it/50.0-1.0);
  timer.stop();
  cout << timer << "\t(computed result = " << yf << ")" << endl;
  fResults << '\t' << timer.value();

  //Atan
  cout << setw(10) << "x" << '\t'
    << setw(10) << "atan(double x)" << '\t'
    << setw(10) << "fastATan(float x)" << '\t'
    << setw(10) << "fastATan(double x)" << endl;
  for(unsigned int i = 0 ; i<sizeof(valuesTrigo)/sizeof(double) ; ++i)
    cout << setw(10) << valuesTrigo[i] << '\t'
         << setw(10) << atan(valuesTrigo[i]) << '\t'
         << setw(10) << FastMath::fastATan(static_cast<float>(valuesTrigo[i])) << '\t'
         << setw(10) << FastMath::fastATan(valuesTrigo[i]) << endl;

  fResults << "\natan";
  cout << "Benchmark atanf(float) (" << taille << " elements)" << endl;
  timer.start();  
  yf = 0;
  for(vector<float>::const_iterator it = vf.begin() ; it != vf.end() ; ++it)
    yf += atanf(*it/50.0f-1.0f);
  timer.stop();
  cout << timer << "\t(computed result = " << yf << ")" << endl;
  fResults << '\t' << timer.value();
  
  cout << "Benchmark atan(double) (" << taille << " elements)" << endl;
  timer.start();  
  yd = 0;
  for(vector<double>::const_iterator it = vd.begin() ; it != vd.end() ; ++it)
    yd += atan(*it/50.0-1.0);
  timer.stop();
  cout << timer << "\t(computed result = " << yd << ")" << endl;
  fResults << '\t' << timer.value();
  
  cout << "Benchmark fastATan(float) (" << taille << " elements)" << endl;
  timer.start();  
  yf = 0;
  for(vector<float>::const_iterator it = vf.begin() ; it != vf.end() ; ++it)
    yf += FastMath::fastATan(*it/50.0f-1.0f);
  timer.stop();
  cout << timer << "\t(computed result = " << yf << ")" << endl;
  fResults << '\t' << timer.value();
  
  cout << "Benchmark fastATan(double) (" << taille << " elements)" << endl;
  timer.start();  
  yd = 0;
  for(vector<double>::const_iterator it = vd.begin() ; it != vd.end() ; ++it)
    yd += FastMath::fastATan(*it/50.0-1.0);
  timer.stop();
  cout << timer << "\t(computed result = " << yd << ")" << endl;
  fResults << '\t' << timer.value();
  
  cout << "Benchmark (float)fastATan(double) (" << taille << " elements)" << endl;
  timer.start();  
  yf = 0;
  for(vector<double>::const_iterator it = vd.begin() ; it != vd.end() ; ++it)
    yf += (float)FastMath::fastATan(*it/50.0-1.0);
  timer.stop();
  cout << timer << "\t(computed result = " << yf << ")" << endl;
  fResults << '\t' << timer.value();

  //Exp
  cout << setw(10) << "x" << '\t'
       << setw(10) << "exp(double x)" << '\t'
       << setw(10) << "fastExp(float x)" << '\t'
       << setw(10) << "fastExp(double x)" << endl;
  for(unsigned int i = 0 ; i<sizeof(values)/sizeof(double) ; ++i)
    cout << setw(10) << values[i] << '\t'
         << setw(10) << exp(values[i]) << '\t'
         << setw(10) << FastMath::fastExp(static_cast<float>(values[i])) << '\t'
         << setw(10) << FastMath::fastExp(values[i]) << endl;

  fResults << "\nexp";
  cout << "Benchmark expf(float) (" << taille << " elements)" << endl;
  timer.start();  
  yf = 0;
  for(vector<float>::const_iterator it = vf.begin() ; it != vf.end() ; ++it)
    yf += expf(*it);
  timer.stop();
  cout << timer << "\t(computed result = " << yf << ")" << endl;
  fResults << '\t' << timer.value();
    
  cout << "Benchmark exp(double) (" << taille << " elements)" << endl;
  timer.start();  
  yd = 0;
  for(vector<double>::const_iterator it = vd.begin() ; it != vd.end() ; ++it)
    yd += exp(*it);
  timer.stop();
  cout << timer << "\t(computed result = " << yd << ")" << endl;
  fResults << '\t' << timer.value();
  
  cout << "Benchmark fastExp(float) (" << taille << " elements)" << endl;
  timer.start();  
  yf = 0;
  for(vector<float>::const_iterator it = vf.begin() ; it != vf.end() ; ++it)
    yf += FastMath::fastExp(*it);
  timer.stop();
  cout << timer << "\t(computed result = " << yf << ")" << endl;
  fResults << '\t' << timer.value();
  
  cout << "Benchmark fastExp(double) (" << taille << " elements)" << endl;
  timer.start();  
  yd = 0;
  for(vector<double>::const_iterator it = vd.begin() ; it != vd.end() ; ++it)
    yd += FastMath::fastExp(*it);
  timer.stop();
  cout << timer << "\t(computed result = " << yd << ")" << endl;
  fResults << '\t' << timer.value();
  
  cout << "Benchmark (float)fastExp(double) (" << taille << " elements)" << endl;
  timer.start();  
  yf = 0;
  for(vector<double>::const_iterator it = vd.begin() ; it != vd.end() ; ++it)
    yf += (float) FastMath::fastExp(*it);
  timer.stop();
  cout << timer << "\t(computed result = " << yf << ")" << endl;
  fResults << '\t' << timer.value();

  //Cosh
  cout << setw(10) << "x" << '\t'
       << setw(10) << "cosh(double x)" << '\t'
       << setw(10) << "fastCosh(float x)" << '\t'
       << setw(10) << "fastCosh(double x)" << endl;
  for(unsigned int i = 0 ; i<sizeof(values)/sizeof(double) ; ++i)
    cout << setw(10) << values[i] << '\t'
         << setw(10) << cosh(values[i]) << '\t'
         << setw(10) << FastMath::fastCosh(static_cast<float>(values[i])) << '\t'
         << setw(10) << FastMath::fastCosh(values[i]) << endl;

  fResults << "\ncosh";
  cout << "Benchmark coshf(float) (" << taille << " elements)" << endl;
  timer.start();  
  yf = 0;
  for(vector<float>::const_iterator it = vf.begin() ; it != vf.end() ; ++it)
    yf += coshf(*it);
  timer.stop();
  cout << timer << "\t(computed result = " << yf << ")" << endl;
  fResults << '\t' << timer.value();
    
  cout << "Benchmark cosh(double) (" << taille << " elements)" << endl;
  timer.start();  
  yd = 0;
  for(vector<double>::const_iterator it = vd.begin() ; it != vd.end() ; ++it)
    yd += cosh(*it);
  timer.stop();
  cout << timer << "\t(computed result = " << yd << ")" << endl;
  fResults << '\t' << timer.value();
  
  cout << "Benchmark fastCosh(float) (" << taille << " elements)" << endl;
  timer.start();  
  yf = 0;
  for(vector<float>::const_iterator it = vf.begin() ; it != vf.end() ; ++it)
    yf += FastMath::fastCosh(*it);
  timer.stop();
  cout << timer << "\t(computed result = " << yf << ")" << endl;
  fResults << '\t' << timer.value();
  
  cout << "Benchmark fastCosh(double) (" << taille << " elements)" << endl;
  timer.start();  
  yd = 0;
  for(vector<double>::const_iterator it = vd.begin() ; it != vd.end() ; ++it)
    yd += FastMath::fastCosh(*it);
  timer.stop();
  cout << timer << "\t(computed result = " << yd << ")" << endl;
  fResults << '\t' << timer.value();
  
  cout << "Benchmark (float)fastCosh(double) (" << taille << " elements)" << endl;
  timer.start();  
  yf = 0;
  for(vector<double>::const_iterator it = vd.begin() ; it != vd.end() ; ++it)
    yf += (float) FastMath::fastCosh(*it);
  timer.stop();
  cout << timer << "\t(computed result = " << yf << ")" << endl;
  fResults << '\t' << timer.value();

  //Sinh
  cout << setw(10) << "x" << '\t'
       << setw(10) << "sinh(double x)" << '\t'
       << setw(10) << "fastSinh(float x)" << '\t'
       << setw(10) << "fastSinh(double x)" << endl;
  for(unsigned int i = 0 ; i<sizeof(values)/sizeof(double) ; ++i)
    cout << setw(10) << values[i] << '\t'
         << setw(10) << sinh(values[i]) << '\t'
         << setw(10) << FastMath::fastSinh(static_cast<float>(values[i])) << '\t'
         << setw(10) << FastMath::fastSinh(values[i]) << endl;

  fResults << "\nsinh";
  cout << "Benchmark sinhf(float) (" << taille << " elements)" << endl;
  timer.start();  
  yf = 0;
  for(vector<float>::const_iterator it = vf.begin() ; it != vf.end() ; ++it)
    yf += sinhf(*it);
  timer.stop();
  cout << timer << "\t(computed result = " << yf << ")" << endl;
  fResults << '\t' << timer.value();
    
  cout << "Benchmark sinh(double) (" << taille << " elements)" << endl;
  timer.start();  
  yd = 0;
  for(vector<double>::const_iterator it = vd.begin() ; it != vd.end() ; ++it)
    yd += sinh(*it);
  timer.stop();
  cout << timer << "\t(computed result = " << yd << ")" << endl;
  fResults << '\t' << timer.value();
  
  cout << "Benchmark fastSinh(float) (" << taille << " elements)" << endl;
  timer.start();  
  yf = 0;
  for(vector<float>::const_iterator it = vf.begin() ; it != vf.end() ; ++it)
    yf += FastMath::fastSinh(*it);
  timer.stop();
  cout << timer << "\t(computed result = " << yf << ")" << endl;
  fResults << '\t' << timer.value();
  
  cout << "Benchmark fastSinh(double) (" << taille << " elements)" << endl;
  timer.start();  
  yd = 0;
  for(vector<double>::const_iterator it = vd.begin() ; it != vd.end() ; ++it)
    yd += FastMath::fastSinh(*it);
  timer.stop();
  cout << timer << "\t(computed result = " << yd << ")" << endl;
  fResults << '\t' << timer.value();
  
  cout << "Benchmark (float)fastSinh(double) (" << taille << " elements)" << endl;
  timer.start();  
  yf = 0;
  for(vector<double>::const_iterator it = vd.begin() ; it != vd.end() ; ++it)
    yf += (float) FastMath::fastSinh(*it);
  timer.stop();
  cout << timer << "\t(computed result = " << yf << ")" << endl;
  fResults << '\t' << timer.value();
  
  //Tanh
  cout << setw(10) << "x" << '\t'
       << setw(10) << "tanh(double x)" << '\t'
       << setw(10) << "fastTanh(float x)" << '\t'
       << setw(10) << "fastTanh(double x)" << endl;
  for(unsigned int i = 0 ; i<sizeof(values)/sizeof(double) ; ++i)
    cout << setw(10) << values[i] << '\t'
         << setw(10) << tanh(values[i]) << '\t'
         << setw(10) << FastMath::fastTanh(static_cast<float>(values[i])) << '\t'
         << setw(10) << FastMath::fastTanh(values[i]) << endl;

  fResults << "\ntanh";
  cout << "Benchmark tanhf(float) (" << taille << " elements)" << endl;
  timer.start();  
  yf = 0;
  for(vector<float>::const_iterator it = vf.begin() ; it != vf.end() ; ++it)
    yf += tanhf(*it);
  timer.stop();
  cout << timer << "\t(computed result = " << yf << ")" << endl;
  fResults << '\t' << timer.value();
    
  cout << "Benchmark tanh(double) (" << taille << " elements)" << endl;
  timer.start();  
  yd = 0;
  for(vector<double>::const_iterator it = vd.begin() ; it != vd.end() ; ++it)
    yd += tanh(*it);
  timer.stop();
  cout << timer << "\t(computed result = " << yd << ")" << endl;
  fResults << '\t' << timer.value();
  
  cout << "Benchmark fastTanh(float) (" << taille << " elements)" << endl;
  timer.start();  
  yf = 0;
  for(vector<float>::const_iterator it = vf.begin() ; it != vf.end() ; ++it)
    yf += FastMath::fastTanh(*it);
  timer.stop();
  cout << timer << "\t(computed result = " << yf << ")" << endl;
  fResults << '\t' << timer.value();
  
  cout << "Benchmark fastTanh(double) (" << taille << " elements)" << endl;
  timer.start();  
  yd = 0;
  for(vector<double>::const_iterator it = vd.begin() ; it != vd.end() ; ++it)
    yd += FastMath::fastTanh(*it);
  timer.stop();
  cout << timer << "\t(computed result = " << yd << ")" << endl;
  fResults << '\t' << timer.value();
  
  cout << "Benchmark (float)fastTanh(double) (" << taille << " elements)" << endl;
  timer.start();  
  yf = 0;
  for(vector<double>::const_iterator it = vd.begin() ; it != vd.end() ; ++it)
    yf += (float) FastMath::fastTanh(*it);
  timer.stop();
  cout << timer << "\t(computed result = " << yf << ")" << endl;
  fResults << '\t' << timer.value();
  
  return 0;
}
